#!/usr/bin/env node

/*! wolke-next-cdk v0.22.0 */
'use strict';

var _filterInstanceProperty = require('@babel/runtime-corejs3/core-js-stable/instance/filter');
var _JSON$stringify = require('@babel/runtime-corejs3/core-js-stable/json/stringify');
var path = require('path');
var fs = require('fs');
var assert = require('assert');
var cdk = require('@aws-cdk/core');
var dotenv = require('universal-dotenv');
var lambda = require('@aws-cdk/aws-lambda');
var s3deploy = require('@aws-cdk/aws-s3-deployment');
var apigateway = require('@aws-cdk/aws-apigatewayv2');
var apigatewayIntegration = require('@aws-cdk/aws-apigatewayv2-integrations');
var events = require('@aws-cdk/aws-events');
var eventsTargets = require('@aws-cdk/aws-events-targets');
var _Date$now = require('@babel/runtime-corejs3/core-js-stable/date/now');
var cr = require('@aws-cdk/custom-resources');
var iam = require('@aws-cdk/aws-iam');
var _Promise = require('@babel/runtime-corejs3/core-js-stable/promise');
var cert = require('@aws-cdk/aws-certificatemanager');
var ssm = require('@aws-cdk/aws-ssm');
var cloudfront = require('@aws-cdk/aws-cloudfront');
var origins = require('@aws-cdk/aws-cloudfront-origins');
var s3 = require('@aws-cdk/aws-s3');
var _forEachInstanceProperty = require('@babel/runtime-corejs3/core-js-stable/instance/for-each');
var _Object$keys = require('@babel/runtime-corejs3/core-js-stable/object/keys');
var _endsWithInstanceProperty = require('@babel/runtime-corejs3/core-js-stable/instance/ends-with');
var util = require('util');
var _includesInstanceProperty = require('@babel/runtime-corejs3/core-js-stable/instance/includes');
var fs$1 = require('fs-extra');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e['default'] : e; }

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      n[k] = e[k];
    });
  }
  n['default'] = e;
  return Object.freeze(n);
}

var _filterInstanceProperty__default = /*#__PURE__*/_interopDefaultLegacy(_filterInstanceProperty);
var _JSON$stringify__default = /*#__PURE__*/_interopDefaultLegacy(_JSON$stringify);
var path__namespace = /*#__PURE__*/_interopNamespace(path);
var fs__namespace = /*#__PURE__*/_interopNamespace(fs);
var assert__namespace = /*#__PURE__*/_interopNamespace(assert);
var cdk__namespace = /*#__PURE__*/_interopNamespace(cdk);
var dotenv__namespace = /*#__PURE__*/_interopNamespace(dotenv);
var lambda__namespace = /*#__PURE__*/_interopNamespace(lambda);
var s3deploy__namespace = /*#__PURE__*/_interopNamespace(s3deploy);
var apigateway__namespace = /*#__PURE__*/_interopNamespace(apigateway);
var apigatewayIntegration__namespace = /*#__PURE__*/_interopNamespace(apigatewayIntegration);
var events__namespace = /*#__PURE__*/_interopNamespace(events);
var eventsTargets__namespace = /*#__PURE__*/_interopNamespace(eventsTargets);
var _Date$now__default = /*#__PURE__*/_interopDefaultLegacy(_Date$now);
var cr__namespace = /*#__PURE__*/_interopNamespace(cr);
var iam__namespace = /*#__PURE__*/_interopNamespace(iam);
var _Promise__default = /*#__PURE__*/_interopDefaultLegacy(_Promise);
var cert__namespace = /*#__PURE__*/_interopNamespace(cert);
var ssm__namespace = /*#__PURE__*/_interopNamespace(ssm);
var cloudfront__namespace = /*#__PURE__*/_interopNamespace(cloudfront);
var origins__namespace = /*#__PURE__*/_interopNamespace(origins);
var s3__namespace = /*#__PURE__*/_interopNamespace(s3);
var _forEachInstanceProperty__default = /*#__PURE__*/_interopDefaultLegacy(_forEachInstanceProperty);
var _Object$keys__default = /*#__PURE__*/_interopDefaultLegacy(_Object$keys);
var _endsWithInstanceProperty__default = /*#__PURE__*/_interopDefaultLegacy(_endsWithInstanceProperty);
var _includesInstanceProperty__default = /*#__PURE__*/_interopDefaultLegacy(_includesInstanceProperty);
var fs__namespace$1 = /*#__PURE__*/_interopNamespace(fs$1);

class SSMParameterReader extends cr__namespace.AwsCustomResource {
  constructor(scope, name, props) {
    const {
      parameterName,
      region
    } = props;
    const ssmAwsSdkCall = {
      service: "SSM",
      action: "getParameter",
      parameters: {
        Name: parameterName
      },
      region,
      physicalResourceId: {
        id: _Date$now__default().toString()
      } // Update physical id to always fetch the latest version

    };
    const policyStatement = new iam__namespace.PolicyStatement({
      actions: ["*"],
      resources: ["*"]
    });
    super(scope, name, {
      onUpdate: ssmAwsSdkCall,
      policy: {
        statements: [policyStatement]
      }
    });
  }

  getParameterValue() {
    return this.getResponseField("Parameter.Value");
  }

}

class NextLambdaGenerator extends cdk__namespace.Construct {
  constructor(scope, id, pathToNextJS) {
    super(scope, id);
    this.nextjsPath = path__namespace.resolve(__dirname, "..", pathToNextJS);
  }
  /*
    patchJavascript(code: string): string {
      return code.replace(/require\("(.+)"\)/, (match, part1) => {
        const inject = fs.readFileSync(
          path.join(__dirname, "..", "lambda", part1),
          "utf8"
        )
        return inject
      })
    }
  */


  async code(staticBucketName, assetBucketName) {
    const entryFile = path__namespace.join(require.resolve("wolke-lambda-handler"));
    const wolkeNextConfig = {
      staticBucketName,
      assetBucketName,
      buildTime: new Date().toUTCString()
    };
    await _Promise__default.all([fs.promises.copyFile(entryFile, path__namespace.join(this.nextjsPath, "next-handler.js")), fs.promises.writeFile(path__namespace.join(this.nextjsPath, "wolke-next-config.json"), _JSON$stringify__default(wolkeNextConfig, null, 2))]);
    return lambda__namespace.Code.fromAsset(this.nextjsPath);
  }

}

/* eslint-disable no-new */
const CERTIFICATE_ARN_SSM_PARAM = "CERTIFICATE_ARN_SSM_PARAM";
function buildCertificateStack(scope, id, config, props) {
  const stack = new cdk__namespace.Stack(scope, id, props);
  const certificate = new cert__namespace.Certificate(stack, `${config.id}-cert`, {
    domainName: config.domainName,
    subjectAlternativeNames: config.domainAliases,
    validationMethod: cert__namespace.ValidationMethod.DNS
  });
  new cdk__namespace.CfnOutput(stack, "Certificate", {
    value: certificate.certificateArn
  });
  new ssm__namespace.StringParameter(stack, `${config.id}-certificateArnSSMParam`, {
    parameterName: `${CERTIFICATE_ARN_SSM_PARAM}-${config.domainName}`,
    simpleName: true,
    stringValue: certificate.certificateArn
  });
  return stack;
}

// eslint-disable-next-line import/no-unresolved
// import { APIGatewayProxyEventV2, APIGatewayProxyResult } from "aws-lambda"
// dummy entry
const MAIN_DOMAIN = "www.example.com"; // eslint-disable-next-line @typescript-eslint/require-await

async function viewerRequest(event, context) {
  const {
    request
  } = event.Records[0].cf;
  return {
    status: 301,
    headers: {
      location: [{
        key: 'Location',
        value: `https://${MAIN_DOMAIN}${request.uri}`
      }]
    }
  };
}
function patchInlineCode(handler, domain, code) {
  return `'use strict';
    const MAIN_DOMAIN = "${domain}"
    exports.${handler} = ${code}
  `;
}

/* eslint-disable @typescript-eslint/naming-convention */
const YEAR_IN_DAYS = 356;
const CACHE_10_YEARS = cdk__namespace.Duration.days(YEAR_IN_DAYS * 10);

function getDomainName(url) {
  return url.split("//")[1].split("/")[0];
}

function createCloudFront(stack, opts) {
  const oai = new cloudfront__namespace.OriginAccessIdentity(stack, `${opts.id}-OAI`, {
    comment: `OIA for S3 bucket.`
  });
  const oneDayCachePolicy = new cloudfront__namespace.CachePolicy(stack, `${opts.id}-SiteDistribution-cache-day`, {
    defaultTtl: cdk__namespace.Duration.days(1),
    enableAcceptEncodingBrotli: true,
    enableAcceptEncodingGzip: true
  });
  const longCachePolicy = new cloudfront__namespace.CachePolicy(stack, `${opts.id}-SiteDistribution-cache-long`, {
    defaultTtl: CACHE_10_YEARS,
    minTtl: CACHE_10_YEARS,
    maxTtl: CACHE_10_YEARS,
    enableAcceptEncodingBrotli: true,
    enableAcceptEncodingGzip: true
  });
  const noCachePolicy = new cloudfront__namespace.CachePolicy(stack, `${opts.id}-SiteDistribution-cache-no`, {
    defaultTtl: cdk__namespace.Duration.millis(0),
    // allowedMethods: cloudfront.CloudFrontAllowedMethods.ALL,
    enableAcceptEncodingBrotli: true,
    enableAcceptEncodingGzip: true
  });
  const s3OriginRequestPolicy = new cloudfront__namespace.OriginRequestPolicy(stack, `${opts.id}-SiteDistribution-orp`, {
    cookieBehavior: cloudfront__namespace.OriginRequestCookieBehavior.none(),
    headerBehavior: cloudfront__namespace.OriginRequestHeaderBehavior.none(),
    queryStringBehavior: cloudfront__namespace.OriginRequestQueryStringBehavior.none()
  });
  const assetBucketOrigin = new origins__namespace.S3Origin(opts.assetBucket, {
    originAccessIdentity: oai
  });
  const staticBucketOrigin = new origins__namespace.S3Origin(opts.staticBucket, {
    originAccessIdentity: oai
  });
  const defaultOrigin = new origins__namespace.HttpOrigin(getDomainName(opts.httpApi.url));
  const certificate = cert__namespace.Certificate.fromCertificateArn(stack, `${opts.id}-SiteDistribution-cert`, opts.certificateArn);
  const distribution = new cloudfront__namespace.Distribution(stack, `${opts.id}-SiteDistribution`, {
    certificate,
    minimumProtocolVersion: cloudfront__namespace.SecurityPolicyProtocol.TLS_V1_1_2016,
    domainNames: [opts.domainName],
    // FIXME: sslMethod: cloudfront.SSLMethod.SNI,
    defaultBehavior: {
      origin: defaultOrigin
    },
    additionalBehaviors: {
      "static/*": {
        cachePolicy: oneDayCachePolicy,
        origin: assetBucketOrigin,
        originRequestPolicy: s3OriginRequestPolicy
      },
      "favicon.ico": {
        cachePolicy: oneDayCachePolicy,
        origin: assetBucketOrigin,
        originRequestPolicy: s3OriginRequestPolicy
      },
      "_next/static/*": {
        cachePolicy: longCachePolicy,
        origin: staticBucketOrigin,
        originRequestPolicy: s3OriginRequestPolicy
      },
      "api/*": {
        cachePolicy: noCachePolicy,
        origin: defaultOrigin,
        allowedMethods: cloudfront__namespace.AllowedMethods.ALLOW_ALL
      }
    }
  });
  const edgeViewerRequestFnt = new cloudfront__namespace.experimental.EdgeFunction(stack, `${opts.id}-Edge-Fnt`, {
    runtime: lambda__namespace.Runtime.NODEJS_12_X,
    handler: 'index.handler',
    code: lambda__namespace.Code.fromInline(patchInlineCode("handler", opts.domainName, viewerRequest.toString())),
    stackId: `${opts.id}-EdgeStack`
  });
  const redirectDistribution = new cloudfront__namespace.Distribution(stack, `${opts.id}-SiteDistribution-redirect`, {
    certificate,
    minimumProtocolVersion: cloudfront__namespace.SecurityPolicyProtocol.TLS_V1_1_2016,
    domainNames: opts.domainAliases,
    defaultBehavior: {
      origin: defaultOrigin,
      edgeLambdas: [{
        functionVersion: edgeViewerRequestFnt.currentVersion,
        eventType: cloudfront__namespace.LambdaEdgeEventType.VIEWER_REQUEST
      }]
    }
  });
  return {
    distribution,
    redirectDistribution,
    oai
  };
}

function createS3(stack, domainName, id) {
  const staticBucketName = `${domainName}`;
  const staticBucket = new s3__namespace.Bucket(stack, `${id}-staticBucket`, {
    bucketName: staticBucketName,
    blockPublicAccess: s3__namespace.BlockPublicAccess.BLOCK_ALL,
    // Destroy is okay here, as we can always recreate it from source
    removalPolicy: cdk__namespace.RemovalPolicy.DESTROY,
    autoDeleteObjects: true
  });
  const assetBucketName = `${domainName}-assets`;
  const assetBucket = new s3__namespace.Bucket(stack, `${id}-assetBucket`, {
    bucketName: assetBucketName,
    blockPublicAccess: s3__namespace.BlockPublicAccess.BLOCK_ALL,
    // Destroy is okay here, as we can always recreate it from source
    removalPolicy: cdk__namespace.RemovalPolicy.DESTROY,
    autoDeleteObjects: true
  });
  return {
    staticBucket,
    assetBucket,
    staticBucketName,
    assetBucketName
  };
}

/* eslint-disable @typescript-eslint/no-unsafe-assignment */
const LAMBDA_EXECUTION_TIMEOUT_SECS = 15;
async function buildNextJSStack(scope, id, config, props) {
  const stack = new cdk__namespace.Stack(scope, id, props);
  /*
  const stableBuildId = await hashPaths([
    pathUtil.join(config.nextjsBasePath, "public"),
    pathUtil.join(config.nextjsBasePath, ".next")
  ])
  */

  const certificateArnReader = new SSMParameterReader(stack, "certificateArnReader", {
    parameterName: `${CERTIFICATE_ARN_SSM_PARAM}-${config.domainName}`,
    region: "us-east-1"
  });
  const certificateArn = certificateArnReader.getParameterValue();
  new cdk__namespace.CfnOutput(stack, "Certificate", {
    value: certificateArn
  });
  const {
    staticBucket,
    staticBucketName,
    assetBucket,
    assetBucketName
  } = createS3(stack, config.domainName, config.id);
  new cdk__namespace.CfnOutput(stack, "Bucket", {
    value: staticBucket.bucketName
  });
  new cdk__namespace.CfnOutput(stack, "AssetBucket", {
    value: assetBucket.bucketName
  });
  const nextLambda = new NextLambdaGenerator(stack, `${config.id}-next-lambda-handler`, path__namespace.join(config.nextjsBasePath, ".next-clean"));
  const nextHandler = new lambda__namespace.Function(stack, `${config.id}-NextHandler`, {
    runtime: lambda__namespace.Runtime.NODEJS_14_X,
    code: await nextLambda.code(staticBucketName, assetBucketName),
    handler: "next-handler.handler",
    timeout: cdk__namespace.Duration.seconds(LAMBDA_EXECUTION_TIMEOUT_SECS),
    memorySize: 1024
  });
  const rule = new events__namespace.Rule(stack, `${config.id}-next-lambda-schedule-rule`, {
    schedule: events__namespace.Schedule.rate(cdk__namespace.Duration.minutes(5)) // Every 5 minutes

  });
  rule.addTarget(new eventsTargets__namespace.LambdaFunction(nextHandler)); // provisioned concurrency is pretty expensive

  /*
  let alias = new lambda.Alias(stack, 'LiveAlias', {
    aliasName: "live",
    version: nextHandler.currentVersion,
    provisionedConcurrentExecutions: 1
  })
  */

  staticBucket.grantRead(nextHandler);
  assetBucket.grantRead(nextHandler);
  const httpApi = new apigateway__namespace.HttpApi(stack, `${config.id}-HttpApi`, {
    createDefaultStage: true,
    defaultIntegration: new apigatewayIntegration__namespace.LambdaProxyIntegration({
      handler: nextHandler
    })
  }); // CloudFront distribution that provides HTTPS

  const {
    distribution,
    redirectDistribution,
    oai
  } = createCloudFront(stack, {
    domainName: config.domainName,
    domainAliases: config.domainAliases,
    certificateArn,
    assetBucket,
    staticBucket,
    httpApi,
    id: config.id
  });
  new cdk__namespace.CfnOutput(stack, "DistributionId", {
    value: distribution.distributionId
  });
  new cdk__namespace.CfnOutput(stack, "DistributionDomainName", {
    value: distribution.distributionDomainName
  });
  new cdk__namespace.CfnOutput(stack, "RedirectDistributionId", {
    value: redirectDistribution.distributionId
  });
  new cdk__namespace.CfnOutput(stack, "RedirectDistributionDomainName", {
    value: redirectDistribution.distributionDomainName
  });
  staticBucket.grantRead(oai);
  assetBucket.grantRead(oai); // Deploy site contents to S3 bucket

  new s3deploy__namespace.BucketDeployment(stack, `${config.id}-DeployAssetsWithInvalidation`, {
    sources: [s3deploy__namespace.Source.asset(path__namespace.join(config.nextjsBasePath, "public"))],
    destinationBucket: assetBucket,
    distribution,
    cacheControl: [s3deploy__namespace.CacheControl.setPublic(), s3deploy__namespace.CacheControl.maxAge(cdk__namespace.Duration.days(1))] // distributionPaths: ["/*"]

  });
  new s3deploy__namespace.BucketDeployment(stack, `${config.id}-DeployNextStaticWithInvalidation`, {
    sources: [s3deploy__namespace.Source.asset(path__namespace.join(config.nextjsBasePath, ".next-clean"))],
    destinationKeyPrefix: "_next/",
    destinationBucket: staticBucket,
    distribution,
    distributionPaths: ["/", "/*"],
    cacheControl: [s3deploy__namespace.CacheControl.setPublic(), s3deploy__namespace.CacheControl.maxAge(cdk__namespace.Duration.days(365))]
  });
  return stack;
}

const PRERENDER_JSON_VERSION = 3;
const ROUTES_JSON_VERSION = 3;
const readFile = util.promisify(fs__namespace.readFile);

async function readJSON(filename, configFiles) {
  if (configFiles && filename in configFiles) {
    return configFiles[filename];
  }

  const content = await readFile(filename, "utf8");
  return JSON.parse(content);
}

async function generateConfig(basePath, configFiles) {
  var _context, _context2, _context3, _context4, _context5;

  const config = {
    staticPages: {},
    dynamicPages: {},
    dynamicRegexpPages: [],
    dataRoutes: [],
    prerenderRoutes: {},
    headers: []
  };
  const pagesManifestPath = path__namespace.join(basePath, "serverless/pages-manifest.json");
  const routesManifestPath = path__namespace.join(basePath, "routes-manifest.json");
  const prerenderManifestPath = path__namespace.join(basePath, "prerender-manifest.json");
  const [manifest, routes, prerender] = await _Promise__default.all([readJSON(pagesManifestPath, configFiles), readJSON(routesManifestPath, configFiles), readJSON(prerenderManifestPath, configFiles)]);

  _forEachInstanceProperty__default(_context = _Object$keys__default(manifest)).call(_context, pageURL => {
    const pagePath = manifest[pageURL];

    if (_endsWithInstanceProperty__default(pagePath).call(pagePath, ".html")) {
      config.staticPages[pageURL] = pagePath;
    } else if (_endsWithInstanceProperty__default(pagePath).call(pagePath, ".js")) {
      config.dynamicPages[pageURL] = pagePath;
    } else {
      console.error("NO TARGET DETECTED: ", pageURL, pagePath);
    }
  });

  if (routes.version !== ROUTES_JSON_VERSION) {
    console.error(`Routes file has wrong version number (should be 3): ${routes.version}`);
  }

  _forEachInstanceProperty__default(_context2 = routes.dataRoutes).call(_context2, ({
    page,
    dataRouteRegex
  }) => {
    config.dataRoutes.push({
      page,
      dataRouteRegexString: dataRouteRegex
    });
  });

  _forEachInstanceProperty__default(_context3 = routes.dynamicRoutes).call(_context3, ({
    page,
    regex
  }) => {
    config.dynamicRegexpPages.push({
      regexString: regex,
      page
    });
  });

  _forEachInstanceProperty__default(_context4 = routes.headers).call(_context4, ({
    source,
    headers,
    regex
  }) => {
    config.headers.push({
      source: source,
      headers: headers,
      regexString: regex
    });
  });

  if (prerender.version !== PRERENDER_JSON_VERSION) {
    console.error(`Prerender file has wrong version number (should be 3): ${prerender.version}`);
  }

  _forEachInstanceProperty__default(_context5 = _Object$keys__default(prerender.routes)).call(_context5, route => {
    const currentConfig = prerender.routes[route];
    const jsonFilename = `pages${route}.json`;
    config.prerenderRoutes[currentConfig.dataRoute] = jsonFilename;
  });

  return config;
}

async function getConfig(basePath, configFiles) {
  return generateConfig(basePath, configFiles);
}

const filterFunc = (src, dest) => {
  if (_endsWithInstanceProperty__default(src).call(src, ".next")) {
    return true;
  }

  if (_includesInstanceProperty__default(src).call(src, ".next/static")) {
    return true;
  }

  if (_includesInstanceProperty__default(src).call(src, ".next/serverless")) {
    return true;
  }

  if (_endsWithInstanceProperty__default(src).call(src, "BUILD_ID")) {
    return true;
  }

  if (_endsWithInstanceProperty__default(src).call(src, "lambda-hooks.js")) {
    return true;
  }

  return false;
};

async function cleanCopyNext(src, dest) {
  await fs__namespace$1.emptyDir(dest);
  await fs__namespace$1.copy(src, dest, {
    filter: filterFunc
  });
}

var _process$env$NEXTJS_R, _process$env$DOMAIN_A;
dotenv__namespace.init();
assert__namespace.ok(process.env.AWS_ACCOUNT !== undefined, "Please add env varible 'AWS_ACCOUNT'");
assert__namespace.ok(process.env.AWS_REGION !== undefined, "Please add env varible 'AWS_REGION'");
assert__namespace.ok(process.env.DOMAIN !== undefined, "Please add env varible 'DOMAIN'");
assert__namespace.ok(process.env.APP_ROOT !== undefined, "Please add env varible 'APP_ROOT'");
const AWS_ACCOUNT = process.env.AWS_ACCOUNT;
const AWS_REGION = process.env.AWS_REGION;
const DOMAIN = process.env.DOMAIN;
const NEXTJS_ROOT = (_process$env$NEXTJS_R = process.env.NEXTJS_ROOT) != null ? _process$env$NEXTJS_R : ".";
const NEXTJS_BASE_PATH = path__namespace.resolve(process.env.APP_ROOT, NEXTJS_ROOT);
const DOMAIN_ALIAS = (_process$env$DOMAIN_A = process.env.DOMAIN_ALIAS) != null ? _process$env$DOMAIN_A : ""; // CERT has to be in us-east-1

const certRegion = {
  account: AWS_ACCOUNT,
  region: "us-east-1"
};
const prodRegion = {
  account: AWS_ACCOUNT,
  region: AWS_REGION
};

async function checkNextJS(basePath) {
  try {
    const stat = await fs.promises.stat(basePath);

    if (!stat.isDirectory()) {
      console.error("NEXTJS_ROOT is not set correctly");
      process.exit(1);
    }
  } catch (error) {
    console.error("NEXTJS_ROOT is not set correctly", error);
    process.exit(1);
  }

  try {
    const stat = await fs.promises.stat(path__namespace.join(basePath, ".next"));

    if (!stat.isDirectory()) {
      console.error("Please build nextjs first");
      process.exit(1);
    }
  } catch (error) {
    console.error("Please build nextjs first", error);
    process.exit(1);
  }

  try {
    const stat = await fs.promises.stat(path__namespace.join(basePath, "public"));

    if (!stat.isDirectory()) {
      console.error("Please provide /public path in nextjs project");
      process.exit(1);
    }
  } catch (error) {
    console.error("Please provide /public path in nextjs project", error);
    process.exit(1);
  }
}

function domain2id(domain) {
  return domain.replace(/\./g, "-");
}

function processDomainAliases(domainAlias) {
  var _context;

  return _filterInstanceProperty__default(_context = domainAlias.split(",")).call(_context, item => item.length > 0);
}

async function createCdkStack() {
  await checkNextJS(NEXTJS_BASE_PATH);
  const app = new cdk__namespace.App();
  const id = domain2id(DOMAIN);
  const domainAliases = processDomainAliases(DOMAIN_ALIAS);
  const nextConfig = await getConfig(path__namespace.join(NEXTJS_BASE_PATH, ".next"));
  await cleanCopyNext(path__namespace.join(NEXTJS_BASE_PATH, ".next"), path__namespace.join(NEXTJS_BASE_PATH, ".next-clean"));
  await fs.promises.writeFile(path__namespace.join(NEXTJS_BASE_PATH, ".next-clean", "wolke-next-routes.json"), _JSON$stringify__default(nextConfig, null, 2));
  /* Certificate */

  const certStack = buildCertificateStack(app, `${id}-CertificateStack`, {
    id,
    domainName: DOMAIN,
    domainAliases,
    nextConfig
  }, {
    env: certRegion
  });
  /* App deployment */

  const nextStack = await buildNextJSStack(app, `${id}-NextStack`, {
    id,
    domainName: DOMAIN,
    nextjsBasePath: NEXTJS_BASE_PATH,
    domainAliases,
    nextConfig
  }, {
    env: prodRegion
  });
  nextStack.addDependency(certStack, "Need to create CERT first");
  return {
    certStack,
    nextStack,
    app,
    id,
    nextConfig
  };
}

/* eslint-disable unicorn/filename-case */
createCdkStack().catch(err => console.error(err.message));
//# sourceMappingURL=next.js.map
