/*! wolke-next-cdk v0.22.0 */
import _filterInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/filter';
import _JSON$stringify from '@babel/runtime-corejs3/core-js-stable/json/stringify';
import * as path from 'path';
import * as fs from 'fs';
import { promises } from 'fs';
import * as assert from 'assert';
import * as cdk from '@aws-cdk/core';
import * as dotenv from 'universal-dotenv';
import * as lambda from '@aws-cdk/aws-lambda';
import * as s3deploy from '@aws-cdk/aws-s3-deployment';
import * as apigateway from '@aws-cdk/aws-apigatewayv2';
import * as apigatewayIntegration from '@aws-cdk/aws-apigatewayv2-integrations';
import * as events from '@aws-cdk/aws-events';
import * as eventsTargets from '@aws-cdk/aws-events-targets';
import _Date$now from '@babel/runtime-corejs3/core-js-stable/date/now';
import * as cr from '@aws-cdk/custom-resources';
import * as iam from '@aws-cdk/aws-iam';
import _Promise from '@babel/runtime-corejs3/core-js-stable/promise';
import * as cert from '@aws-cdk/aws-certificatemanager';
import * as ssm from '@aws-cdk/aws-ssm';
import * as cloudfront from '@aws-cdk/aws-cloudfront';
import * as origins from '@aws-cdk/aws-cloudfront-origins';
import * as s3 from '@aws-cdk/aws-s3';
import _forEachInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/for-each';
import _Object$keys from '@babel/runtime-corejs3/core-js-stable/object/keys';
import _endsWithInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/ends-with';
import { promisify } from 'util';
import _includesInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/includes';
import * as fs$1 from 'fs-extra';

class SSMParameterReader extends cr.AwsCustomResource {
  constructor(scope, name, props) {
    const {
      parameterName,
      region
    } = props;
    const ssmAwsSdkCall = {
      service: "SSM",
      action: "getParameter",
      parameters: {
        Name: parameterName
      },
      region,
      physicalResourceId: {
        id: _Date$now().toString()
      } // Update physical id to always fetch the latest version

    };
    const policyStatement = new iam.PolicyStatement({
      actions: ["*"],
      resources: ["*"]
    });
    super(scope, name, {
      onUpdate: ssmAwsSdkCall,
      policy: {
        statements: [policyStatement]
      }
    });
  }

  getParameterValue() {
    return this.getResponseField("Parameter.Value");
  }

}

class NextLambdaGenerator extends cdk.Construct {
  constructor(scope, id, pathToNextJS) {
    super(scope, id);
    this.nextjsPath = path.resolve(__dirname, "..", pathToNextJS);
  }
  /*
    patchJavascript(code: string): string {
      return code.replace(/require\("(.+)"\)/, (match, part1) => {
        const inject = fs.readFileSync(
          path.join(__dirname, "..", "lambda", part1),
          "utf8"
        )
        return inject
      })
    }
  */


  async code(staticBucketName, assetBucketName) {
    const entryFile = path.join(require.resolve("wolke-lambda-handler"));
    const wolkeNextConfig = {
      staticBucketName,
      assetBucketName,
      buildTime: new Date().toUTCString()
    };
    await _Promise.all([promises.copyFile(entryFile, path.join(this.nextjsPath, "next-handler.js")), promises.writeFile(path.join(this.nextjsPath, "wolke-next-config.json"), _JSON$stringify(wolkeNextConfig, null, 2))]);
    return lambda.Code.fromAsset(this.nextjsPath);
  }

}

/* eslint-disable no-new */
const CERTIFICATE_ARN_SSM_PARAM = "CERTIFICATE_ARN_SSM_PARAM";
function buildCertificateStack(scope, id, config, props) {
  const stack = new cdk.Stack(scope, id, props);
  const certificate = new cert.Certificate(stack, `${config.id}-cert`, {
    domainName: config.domainName,
    subjectAlternativeNames: config.domainAliases,
    validationMethod: cert.ValidationMethod.DNS
  });
  new cdk.CfnOutput(stack, "Certificate", {
    value: certificate.certificateArn
  });
  new ssm.StringParameter(stack, `${config.id}-certificateArnSSMParam`, {
    parameterName: `${CERTIFICATE_ARN_SSM_PARAM}-${config.domainName}`,
    simpleName: true,
    stringValue: certificate.certificateArn
  });
  return stack;
}

// eslint-disable-next-line import/no-unresolved
// import { APIGatewayProxyEventV2, APIGatewayProxyResult } from "aws-lambda"
// dummy entry
const MAIN_DOMAIN = "www.example.com"; // eslint-disable-next-line @typescript-eslint/require-await

async function viewerRequest(event, context) {
  const {
    request
  } = event.Records[0].cf;
  return {
    status: 301,
    headers: {
      location: [{
        key: 'Location',
        value: `https://${MAIN_DOMAIN}${request.uri}`
      }]
    }
  };
}
function patchInlineCode(handler, domain, code) {
  return `'use strict';
    const MAIN_DOMAIN = "${domain}"
    exports.${handler} = ${code}
  `;
}

/* eslint-disable @typescript-eslint/naming-convention */
const YEAR_IN_DAYS = 356;
const CACHE_10_YEARS = cdk.Duration.days(YEAR_IN_DAYS * 10);

function getDomainName(url) {
  return url.split("//")[1].split("/")[0];
}

function createCloudFront(stack, opts) {
  const oai = new cloudfront.OriginAccessIdentity(stack, `${opts.id}-OAI`, {
    comment: `OIA for S3 bucket.`
  });
  const oneDayCachePolicy = new cloudfront.CachePolicy(stack, `${opts.id}-SiteDistribution-cache-day`, {
    defaultTtl: cdk.Duration.days(1),
    enableAcceptEncodingBrotli: true,
    enableAcceptEncodingGzip: true
  });
  const longCachePolicy = new cloudfront.CachePolicy(stack, `${opts.id}-SiteDistribution-cache-long`, {
    defaultTtl: CACHE_10_YEARS,
    minTtl: CACHE_10_YEARS,
    maxTtl: CACHE_10_YEARS,
    enableAcceptEncodingBrotli: true,
    enableAcceptEncodingGzip: true
  });
  const noCachePolicy = new cloudfront.CachePolicy(stack, `${opts.id}-SiteDistribution-cache-no`, {
    defaultTtl: cdk.Duration.millis(0),
    // allowedMethods: cloudfront.CloudFrontAllowedMethods.ALL,
    enableAcceptEncodingBrotli: true,
    enableAcceptEncodingGzip: true
  });
  const s3OriginRequestPolicy = new cloudfront.OriginRequestPolicy(stack, `${opts.id}-SiteDistribution-orp`, {
    cookieBehavior: cloudfront.OriginRequestCookieBehavior.none(),
    headerBehavior: cloudfront.OriginRequestHeaderBehavior.none(),
    queryStringBehavior: cloudfront.OriginRequestQueryStringBehavior.none()
  });
  const assetBucketOrigin = new origins.S3Origin(opts.assetBucket, {
    originAccessIdentity: oai
  });
  const staticBucketOrigin = new origins.S3Origin(opts.staticBucket, {
    originAccessIdentity: oai
  });
  const defaultOrigin = new origins.HttpOrigin(getDomainName(opts.httpApi.url));
  const certificate = cert.Certificate.fromCertificateArn(stack, `${opts.id}-SiteDistribution-cert`, opts.certificateArn);
  const distribution = new cloudfront.Distribution(stack, `${opts.id}-SiteDistribution`, {
    certificate,
    minimumProtocolVersion: cloudfront.SecurityPolicyProtocol.TLS_V1_1_2016,
    domainNames: [opts.domainName],
    // FIXME: sslMethod: cloudfront.SSLMethod.SNI,
    defaultBehavior: {
      origin: defaultOrigin
    },
    additionalBehaviors: {
      "static/*": {
        cachePolicy: oneDayCachePolicy,
        origin: assetBucketOrigin,
        originRequestPolicy: s3OriginRequestPolicy
      },
      "favicon.ico": {
        cachePolicy: oneDayCachePolicy,
        origin: assetBucketOrigin,
        originRequestPolicy: s3OriginRequestPolicy
      },
      "_next/static/*": {
        cachePolicy: longCachePolicy,
        origin: staticBucketOrigin,
        originRequestPolicy: s3OriginRequestPolicy
      },
      "api/*": {
        cachePolicy: noCachePolicy,
        origin: defaultOrigin,
        allowedMethods: cloudfront.AllowedMethods.ALLOW_ALL
      }
    }
  });
  const edgeViewerRequestFnt = new cloudfront.experimental.EdgeFunction(stack, `${opts.id}-Edge-Fnt`, {
    runtime: lambda.Runtime.NODEJS_12_X,
    handler: 'index.handler',
    code: lambda.Code.fromInline(patchInlineCode("handler", opts.domainName, viewerRequest.toString())),
    stackId: `${opts.id}-EdgeStack`
  });
  const redirectDistribution = new cloudfront.Distribution(stack, `${opts.id}-SiteDistribution-redirect`, {
    certificate,
    minimumProtocolVersion: cloudfront.SecurityPolicyProtocol.TLS_V1_1_2016,
    domainNames: opts.domainAliases,
    defaultBehavior: {
      origin: defaultOrigin,
      edgeLambdas: [{
        functionVersion: edgeViewerRequestFnt.currentVersion,
        eventType: cloudfront.LambdaEdgeEventType.VIEWER_REQUEST
      }]
    }
  });
  return {
    distribution,
    redirectDistribution,
    oai
  };
}

function createS3(stack, domainName, id) {
  const staticBucketName = `${domainName}`;
  const staticBucket = new s3.Bucket(stack, `${id}-staticBucket`, {
    bucketName: staticBucketName,
    blockPublicAccess: s3.BlockPublicAccess.BLOCK_ALL,
    // Destroy is okay here, as we can always recreate it from source
    removalPolicy: cdk.RemovalPolicy.DESTROY,
    autoDeleteObjects: true
  });
  const assetBucketName = `${domainName}-assets`;
  const assetBucket = new s3.Bucket(stack, `${id}-assetBucket`, {
    bucketName: assetBucketName,
    blockPublicAccess: s3.BlockPublicAccess.BLOCK_ALL,
    // Destroy is okay here, as we can always recreate it from source
    removalPolicy: cdk.RemovalPolicy.DESTROY,
    autoDeleteObjects: true
  });
  return {
    staticBucket,
    assetBucket,
    staticBucketName,
    assetBucketName
  };
}

/* eslint-disable @typescript-eslint/no-unsafe-assignment */
const LAMBDA_EXECUTION_TIMEOUT_SECS = 15;
async function buildNextJSStack(scope, id, config, props) {
  const stack = new cdk.Stack(scope, id, props);
  /*
  const stableBuildId = await hashPaths([
    pathUtil.join(config.nextjsBasePath, "public"),
    pathUtil.join(config.nextjsBasePath, ".next")
  ])
  */

  const certificateArnReader = new SSMParameterReader(stack, "certificateArnReader", {
    parameterName: `${CERTIFICATE_ARN_SSM_PARAM}-${config.domainName}`,
    region: "us-east-1"
  });
  const certificateArn = certificateArnReader.getParameterValue();
  new cdk.CfnOutput(stack, "Certificate", {
    value: certificateArn
  });
  const {
    staticBucket,
    staticBucketName,
    assetBucket,
    assetBucketName
  } = createS3(stack, config.domainName, config.id);
  new cdk.CfnOutput(stack, "Bucket", {
    value: staticBucket.bucketName
  });
  new cdk.CfnOutput(stack, "AssetBucket", {
    value: assetBucket.bucketName
  });
  const nextLambda = new NextLambdaGenerator(stack, `${config.id}-next-lambda-handler`, path.join(config.nextjsBasePath, ".next-clean"));
  const nextHandler = new lambda.Function(stack, `${config.id}-NextHandler`, {
    runtime: lambda.Runtime.NODEJS_14_X,
    code: await nextLambda.code(staticBucketName, assetBucketName),
    handler: "next-handler.handler",
    timeout: cdk.Duration.seconds(LAMBDA_EXECUTION_TIMEOUT_SECS),
    memorySize: 1024
  });
  const rule = new events.Rule(stack, `${config.id}-next-lambda-schedule-rule`, {
    schedule: events.Schedule.rate(cdk.Duration.minutes(5)) // Every 5 minutes

  });
  rule.addTarget(new eventsTargets.LambdaFunction(nextHandler)); // provisioned concurrency is pretty expensive

  /*
  let alias = new lambda.Alias(stack, 'LiveAlias', {
    aliasName: "live",
    version: nextHandler.currentVersion,
    provisionedConcurrentExecutions: 1
  })
  */

  staticBucket.grantRead(nextHandler);
  assetBucket.grantRead(nextHandler);
  const httpApi = new apigateway.HttpApi(stack, `${config.id}-HttpApi`, {
    createDefaultStage: true,
    defaultIntegration: new apigatewayIntegration.LambdaProxyIntegration({
      handler: nextHandler
    })
  }); // CloudFront distribution that provides HTTPS

  const {
    distribution,
    redirectDistribution,
    oai
  } = createCloudFront(stack, {
    domainName: config.domainName,
    domainAliases: config.domainAliases,
    certificateArn,
    assetBucket,
    staticBucket,
    httpApi,
    id: config.id
  });
  new cdk.CfnOutput(stack, "DistributionId", {
    value: distribution.distributionId
  });
  new cdk.CfnOutput(stack, "DistributionDomainName", {
    value: distribution.distributionDomainName
  });
  new cdk.CfnOutput(stack, "RedirectDistributionId", {
    value: redirectDistribution.distributionId
  });
  new cdk.CfnOutput(stack, "RedirectDistributionDomainName", {
    value: redirectDistribution.distributionDomainName
  });
  staticBucket.grantRead(oai);
  assetBucket.grantRead(oai); // Deploy site contents to S3 bucket

  new s3deploy.BucketDeployment(stack, `${config.id}-DeployAssetsWithInvalidation`, {
    sources: [s3deploy.Source.asset(path.join(config.nextjsBasePath, "public"))],
    destinationBucket: assetBucket,
    distribution,
    cacheControl: [s3deploy.CacheControl.setPublic(), s3deploy.CacheControl.maxAge(cdk.Duration.days(1))] // distributionPaths: ["/*"]

  });
  new s3deploy.BucketDeployment(stack, `${config.id}-DeployNextStaticWithInvalidation`, {
    sources: [s3deploy.Source.asset(path.join(config.nextjsBasePath, ".next-clean"))],
    destinationKeyPrefix: "_next/",
    destinationBucket: staticBucket,
    distribution,
    distributionPaths: ["/", "/*"],
    cacheControl: [s3deploy.CacheControl.setPublic(), s3deploy.CacheControl.maxAge(cdk.Duration.days(365))]
  });
  return stack;
}

const PRERENDER_JSON_VERSION = 3;
const ROUTES_JSON_VERSION = 3;
const readFile = promisify(fs.readFile);

async function readJSON(filename, configFiles) {
  if (configFiles && filename in configFiles) {
    return configFiles[filename];
  }

  const content = await readFile(filename, "utf8");
  return JSON.parse(content);
}

async function generateConfig(basePath, configFiles) {
  var _context, _context2, _context3, _context4, _context5;

  const config = {
    staticPages: {},
    dynamicPages: {},
    dynamicRegexpPages: [],
    dataRoutes: [],
    prerenderRoutes: {},
    headers: []
  };
  const pagesManifestPath = path.join(basePath, "serverless/pages-manifest.json");
  const routesManifestPath = path.join(basePath, "routes-manifest.json");
  const prerenderManifestPath = path.join(basePath, "prerender-manifest.json");
  const [manifest, routes, prerender] = await _Promise.all([readJSON(pagesManifestPath, configFiles), readJSON(routesManifestPath, configFiles), readJSON(prerenderManifestPath, configFiles)]);

  _forEachInstanceProperty(_context = _Object$keys(manifest)).call(_context, pageURL => {
    const pagePath = manifest[pageURL];

    if (_endsWithInstanceProperty(pagePath).call(pagePath, ".html")) {
      config.staticPages[pageURL] = pagePath;
    } else if (_endsWithInstanceProperty(pagePath).call(pagePath, ".js")) {
      config.dynamicPages[pageURL] = pagePath;
    } else {
      console.error("NO TARGET DETECTED: ", pageURL, pagePath);
    }
  });

  if (routes.version !== ROUTES_JSON_VERSION) {
    console.error(`Routes file has wrong version number (should be 3): ${routes.version}`);
  }

  _forEachInstanceProperty(_context2 = routes.dataRoutes).call(_context2, ({
    page,
    dataRouteRegex
  }) => {
    config.dataRoutes.push({
      page,
      dataRouteRegexString: dataRouteRegex
    });
  });

  _forEachInstanceProperty(_context3 = routes.dynamicRoutes).call(_context3, ({
    page,
    regex
  }) => {
    config.dynamicRegexpPages.push({
      regexString: regex,
      page
    });
  });

  _forEachInstanceProperty(_context4 = routes.headers).call(_context4, ({
    source,
    headers,
    regex
  }) => {
    config.headers.push({
      source: source,
      headers: headers,
      regexString: regex
    });
  });

  if (prerender.version !== PRERENDER_JSON_VERSION) {
    console.error(`Prerender file has wrong version number (should be 3): ${prerender.version}`);
  }

  _forEachInstanceProperty(_context5 = _Object$keys(prerender.routes)).call(_context5, route => {
    const currentConfig = prerender.routes[route];
    const jsonFilename = `pages${route}.json`;
    config.prerenderRoutes[currentConfig.dataRoute] = jsonFilename;
  });

  return config;
}

async function getConfig(basePath, configFiles) {
  return generateConfig(basePath, configFiles);
}

const filterFunc = (src, dest) => {
  if (_endsWithInstanceProperty(src).call(src, ".next")) {
    return true;
  }

  if (_includesInstanceProperty(src).call(src, ".next/static")) {
    return true;
  }

  if (_includesInstanceProperty(src).call(src, ".next/serverless")) {
    return true;
  }

  if (_endsWithInstanceProperty(src).call(src, "BUILD_ID")) {
    return true;
  }

  if (_endsWithInstanceProperty(src).call(src, "lambda-hooks.js")) {
    return true;
  }

  return false;
};

async function cleanCopyNext(src, dest) {
  await fs$1.emptyDir(dest);
  await fs$1.copy(src, dest, {
    filter: filterFunc
  });
}

var _process$env$NEXTJS_R, _process$env$DOMAIN_A;
dotenv.init();
assert.ok(process.env.AWS_ACCOUNT !== undefined, "Please add env varible 'AWS_ACCOUNT'");
assert.ok(process.env.AWS_REGION !== undefined, "Please add env varible 'AWS_REGION'");
assert.ok(process.env.DOMAIN !== undefined, "Please add env varible 'DOMAIN'");
assert.ok(process.env.APP_ROOT !== undefined, "Please add env varible 'APP_ROOT'");
const AWS_ACCOUNT = process.env.AWS_ACCOUNT;
const AWS_REGION = process.env.AWS_REGION;
const DOMAIN = process.env.DOMAIN;
const NEXTJS_ROOT = (_process$env$NEXTJS_R = process.env.NEXTJS_ROOT) != null ? _process$env$NEXTJS_R : ".";
const NEXTJS_BASE_PATH = path.resolve(process.env.APP_ROOT, NEXTJS_ROOT);
const DOMAIN_ALIAS = (_process$env$DOMAIN_A = process.env.DOMAIN_ALIAS) != null ? _process$env$DOMAIN_A : ""; // CERT has to be in us-east-1

const certRegion = {
  account: AWS_ACCOUNT,
  region: "us-east-1"
};
const prodRegion = {
  account: AWS_ACCOUNT,
  region: AWS_REGION
};

async function checkNextJS(basePath) {
  try {
    const stat = await promises.stat(basePath);

    if (!stat.isDirectory()) {
      console.error("NEXTJS_ROOT is not set correctly");
      process.exit(1);
    }
  } catch (error) {
    console.error("NEXTJS_ROOT is not set correctly", error);
    process.exit(1);
  }

  try {
    const stat = await promises.stat(path.join(basePath, ".next"));

    if (!stat.isDirectory()) {
      console.error("Please build nextjs first");
      process.exit(1);
    }
  } catch (error) {
    console.error("Please build nextjs first", error);
    process.exit(1);
  }

  try {
    const stat = await promises.stat(path.join(basePath, "public"));

    if (!stat.isDirectory()) {
      console.error("Please provide /public path in nextjs project");
      process.exit(1);
    }
  } catch (error) {
    console.error("Please provide /public path in nextjs project", error);
    process.exit(1);
  }
}

function domain2id(domain) {
  return domain.replace(/\./g, "-");
}

function processDomainAliases(domainAlias) {
  var _context;

  return _filterInstanceProperty(_context = domainAlias.split(",")).call(_context, item => item.length > 0);
}

async function createCdkStack() {
  await checkNextJS(NEXTJS_BASE_PATH);
  const app = new cdk.App();
  const id = domain2id(DOMAIN);
  const domainAliases = processDomainAliases(DOMAIN_ALIAS);
  const nextConfig = await getConfig(path.join(NEXTJS_BASE_PATH, ".next"));
  await cleanCopyNext(path.join(NEXTJS_BASE_PATH, ".next"), path.join(NEXTJS_BASE_PATH, ".next-clean"));
  await promises.writeFile(path.join(NEXTJS_BASE_PATH, ".next-clean", "wolke-next-routes.json"), _JSON$stringify(nextConfig, null, 2));
  /* Certificate */

  const certStack = buildCertificateStack(app, `${id}-CertificateStack`, {
    id,
    domainName: DOMAIN,
    domainAliases,
    nextConfig
  }, {
    env: certRegion
  });
  /* App deployment */

  const nextStack = await buildNextJSStack(app, `${id}-NextStack`, {
    id,
    domainName: DOMAIN,
    nextjsBasePath: NEXTJS_BASE_PATH,
    domainAliases,
    nextConfig
  }, {
    env: prodRegion
  });
  nextStack.addDependency(certStack, "Need to create CERT first");
  return {
    certStack,
    nextStack,
    app,
    id,
    nextConfig
  };
}

export { createCdkStack };
//# sourceMappingURL=index.esm.js.map
